/**
 ** WIMP -- WIesbaden MIps Processor
 **
 ** WIMP memory dump utility
 **
 ** Usage: wump <program.wo> [<program.dump>]
 **
 ** Copyright (c) 2014, Steffen Reith, RheinMain University of Applied Sciences
 ** All rights reserved.
 ** 
 ** Redistribution and use in source and binary forms, with or without modification,
 ** are permitted provided that the following conditions are met:
 **
 ** 1. Redistributions of source code must retain the above copyright notice, this
 **    list of conditions and the following disclaimer.
 ** 2. Redistributions in binary form must reproduce the above copyright notice,
 **    this list of conditions and the following disclaimer in the documentation
 **    and/or other materials provided with the distribution.
 ** 
 ** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 ** AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 ** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 ** ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 ** LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 ** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 ** GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 ** HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 ** LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 ** OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 ** OF SUCH DAMAGE.
 **
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>

/**
 * (mis-)use net functions for optional byte swapping:
 */
#include <netinet/in.h>
#define swapl(x) htonl(x)

#define BITS 32

void convBin(char* str, unsigned long word)
{
  int i;

  /*- Terminate string -*/
  str[BITS] = '\0';

  /*- For all digits -*/
  for (i = 0; i < BITS; i++) {

    /*- Check the ith digit -*/
    if ((word >> i) % 2 == 1) {

      /*- Digit one -*/
      str[BITS -1 - i] = '1';

    } else {

      /*- Digit zero -*/
      str[BITS - 1 - i] = '0';

    }

  } 

}

int main (int argc, char *argv[])
{
  char *infile; 
  char *outfile;

  FILE *in;
  FILE *out;

  char str[BITS + 1];

  unsigned long word;

  /*- Check number of arguments -*/
  if (argc < 2) {

    /*- Print a should usage message */
    (void) fprintf (stderr, "Usage: %s <input file> [<output file>]\n", argv[0]);
    exit(EXIT_SUCCESS);
 
  } else {
      
    /*- Use first argument as input-file */
    infile = argv[1];

  }

  /*- We got a second argument -> use it as output -*/
  if (argc == 3) {        
     
    /*- Use second argument as output name -*/
    outfile = argv[2];

  } else {

    /*- Set default output name -*/
    outfile = "out.dump";

  }

  /*- Open input file -*/
  in = fopen (infile, "r");

  /*- Check for stdout -*/
  if (strcmp(outfile, "-")) {
 
    /*- Open output file -*/
    out = fopen (outfile, "w");

  } else {

    /*- We use stdout for output -*/
    out = stdout;

  }

  /*- Check if we are able to read -*/
  if(in == NULL) {

    /*- Give a short message -*/
    (void) fprintf(stderr, "Cannot open file '%s'\n", infile);
    exit(EXIT_FAILURE);

  }
 
  /*- Check if we are able to read -*/
  if(out == NULL) {

    /*- Give a short message -*/
    (void) fprintf(stderr, "Cannot create file '%s'\n", outfile);
    exit(EXIT_FAILURE);

  }

  /*- Read all 32-bit words -*/
  while (!feof(in)) {

    fread(&word, 4, 1, in);
    
    convBin(str, swapl(word));

    (void) fprintf(out, "%s\n", str); 

  }

  /*- Everything was fine -*/
  return(EXIT_SUCCESS);
}
