/**
 ** WIMP -- WIesbaden MIps Processor
 **
 ** WImp disassembler
 **
 ** Usage: wid <program.wo>
 **
 ** Copyright (c) 2014, Robert Kaiser, RheinMain University of Applied Sciences
 ** All rights reserved.
 ** 
 ** Redistribution and use in source and binary forms, with or without modification,
 ** are permitted provided that the following conditions are met:
 **
 ** 1. Redistributions of source code must retain the above copyright notice, this
 **    list of conditions and the following disclaimer.
 ** 2. Redistributions in binary form must reproduce the above copyright notice,
 **    this list of conditions and the following disclaimer in the documentation
 **    and/or other materials provided with the distribution.
 ** 
 ** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 ** AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 ** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 ** ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 ** LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 ** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 ** GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 ** HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 ** LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 ** OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 ** OF SUCH DAMAGE.
 **
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <assert.h>


/**
 * (mis-)use net functions for optional byte swapping:
 */
#include <netinet/in.h>
#define swapl(x) htonl(x)

#include "wimp-opcodes.h"

#include "wimp-dis.h"

int  verbose    = 0;


/** The Disassembler: */
static void TheDisassembler(FILE *f, int location)
{
    WimpInst_t Instruction;
    char Code[64];
    int OpCode;

    while(fread(&Instruction, sizeof(WimpInst_t), 1, f) == 1)
    {
        /** fetch next instruction */
        Instruction = swapl(Instruction);

        /** get the opcode field */
        OpCode = OPTYPE(Instruction);

        if(OpCode < 0 || OpCode >= NUM_INSTRUCTIONS)
        {
            printf("%08x: %08x dc 0x%x\n", location, Instruction, Instruction);
        }
        else
        {   /** Disassemble instruction */
            Disassemble(Code, Instruction, location + sizeof(Instruction));
            /** if verbose, print binary code too */
            if(verbose)
                printf("%08x: %08x %s\n", location, Instruction, Code);
            else
                printf("%08x: %s\n", location, Code);
        }
        location += sizeof(WimpInst_t);
    }
}

static void rtfm(char* argv[])
{
    printf("Usage: %s {Options} <program file>\n", argv[0]);
    printf("Options:\n");
    printf("    -o <origin>     Assume code to begin at <origin> (default:0)\n");
    printf("    -v              be verbose\n");
}


int main(int argc, char *argv[])
{
    FILE *file;
    char *end;
    int option;
    int origin = 0;

    /**
     * Parse commandline options: see "man getopt"
     */
    while ((option = getopt(argc, argv, "?hvo:")) != -1)
    {
        switch(option)
        {
            case '?':
            case 'h':
                rtfm(argv);
                return EXIT_FAILURE;
                break;
            case 'o':
                origin = strtoul(optarg, &end, 0);
                if (end == optarg)
                {
                    rtfm(argv);
                    return EXIT_FAILURE;
                }

                break;
            case 'v':
                ++verbose;
                break;
        }
    }
    if(argc - optind != 1)
    {
        rtfm(argv);
        return EXIT_FAILURE;
    }
    if(!(file = fopen(argv[optind], "r")))
    {
        perror("open");
        return EXIT_FAILURE;
    }
    TheDisassembler(file, origin);
    return EXIT_SUCCESS;
}

