/**
 ** WIMP -- WIesbaden MIps Processor
 **
 ** WIMP Memory interface
 **
 ** Copyright (c) 2014, Robert Kaiser, RheinMain University of Applied Sciences
 ** All rights reserved.
 ** 
 ** Redistribution and use in source and binary forms, with or without modification,
 ** are permitted provided that the following conditions are met:
 **
 ** 1. Redistributions of source code must retain the above copyright notice, this
 **    list of conditions and the following disclaimer.
 ** 2. Redistributions in binary form must reproduce the above copyright notice,
 **    this list of conditions and the following disclaimer in the documentation
 **    and/or other materials provided with the distribution.
 ** 
 ** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 ** AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 ** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 ** ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 ** LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 ** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 ** GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 ** HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 ** LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 ** OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 ** OF SUCH DAMAGE.
 **
 */

#ifndef __WIMP_MEM_H__
#define __WIMP_MEM_H__ 1

#include <stdint.h>
#include "wimp-opcodes.h"
#include "wimp-io.h"

/** create a new address space */
extern int M_CreateSpace(char *filename, unsigned int size);

/** add memory-mapped I/O device to address space */
extern int M_AddMMIO(int ID, mmio_t *dev);

/** delete an address space */
extern void M_DeleteSpace(int ID);

/** check if address is valid in data memory */
extern int M_DataAccessWord(int Address);

/** read a word from data memory */
extern uint32_t M_ReadDataWord(int Address);

/** write a word to data memory */
extern void M_WriteDataWord(unsigned int Address, uint32_t Data);

/** Check if address is valid in program memory */
extern int M_ProgramAccessWord(unsigned int Address);

/** read a word from program memory */
extern WimpInst_t M_ReadProgramWord(unsigned int Address);

#endif /* __WIMP_MEM_H__ */
