;
; Sieb des Eratosthenes in WIMP Assembler
;
; http://de.wikipedia.org/wiki/Sieb_des_Eratosthenes
;
;
; Registerverwendung:
; s0: N            : Obergrenze fuer den sieve-Algorithmus
; s1: gestrichen[]: char-Array der bereits gestrichenen Primzahlen
; s2 : anzahl      : Anzahl gefundener Primzahlen
; s3 : i           : Laufvariable
; s4 : j           : Laufvariable
; s5 : temp        : Temp-Variable fuer alles Moegliche
; s6 : temp        : Temp-Variable fuer alles Moegliche
;
    la     $sp,stk     ; Stackpointer initialisieren
    li     $s2,0       ; anzahl = 0
    ;
    ; Obergrenze N eingeben
    ;
retry:
    la     $a0,msg1
    jal    puts
    jal    getdec     ; v0 = N
    move   s0,v0
    subiu  s5,v0,400
    bltz   s5,goahead
    la     a0,msg6
    jal    puts
    b      retry
goahead:
    ;
    ; Array gestrichen[] anlegen und mit nullen ("false") fuellen:
    ;
    la     s1,gestrichen
    move   s5,s1
    move   s3,zero      ; i = 0
fill:
    sw     zero,0(s5)   ; (int)gestrichen[i] = 0
    addiu  s5,s5,4      ; i += sizeof(int)
    addiu  s3,s3,4
    ble    s3,s0,fill   ; i = N? nein -> naechstes Word fuellen
    ;
    ; Ausgabe: "Primzahlen von 2 bis <N>\n"
    ;
    la     a0,msg2
    jal    puts
    move   a0,s0     ; N ausgeben
    jal    putdec
    la     a0,msg3
    jal    puts

    ;
    ; Hier beginnt der eigentliche Algorithmus:
    ; - Zahl nicht gestrichen -> ist eine Primzahl 
    ;   -> ausgeben und alle Vielfachen der Zahl streichen
    li     s3,2         ; for(i = 2; i < N; i++)
loop:
    add    s5,s3,s1
    lb     s6,0(s5)     ; if(!gestrichen[i])
    bne    s6,zero,cont ; ueberspringen falls true -> Zahl wurde gestrichen
    addiu  s2,s2,1      ; ++anzahl
    move   a0,s3
    jal    putdec       ; i ausgeben
    la     a0,msg3      ; Zeilenvorschub
    jal    puts
    li     s6,1         ; s6 = 1 ("true")
    ; alle Vielfachen von i streichen:
    sll    s4,s3,1      ; for(j = 2*i; j < N; j += i)
loop1:
    blt    s0,s4,cont   ; j < N?

    add    s5,s4,s1
    sb     s6,0(s5)     ; gestrichen[j] = 1
    add    s4,s4,s3     ; j += i
    b      loop1
cont:
    addiu  s3,s3,1      ; ++i
    blt    s3,s0,loop   ; i = N? nein -> naechstes i pruefen
    ;
    ; Ausgabe: "Insgesamt <anzahl> Primzahlen\n"
    ;
    la     a0,msg4
    jal    puts
    move   a0,s2
    jal    putdec        ; anzahl ausgeben
    la     a0,msg5
    jal    puts
    syscall 0

;------------------------------------------------------------------
; Der Rest sind Hilfsroutinen...
;------------------------------------------------------------------

; Hilfsroutine zur integer-Division:
; a0 = Dividend, a1 = Divisor
; Rückgabe:
; v0 = Quotient, v1 = Rest
; t0: i: zaehler
div:
	beq a1,zero,divbyzero  ; if(d == 0)
	                       ;	return -1;
	li v0,0           ; q = 0;
	li v1,0           ; r = 0;
	li t0,31          ; for(i = 31; i >= 0; i--)
__div1:               ; {
    sll v1,v1,1       ;	  r <<= 1;
    li   t1,1
    sllv t1,t1,t0
    and  t2,a0,t1
    srlv t2,t2,t0
    or   v1,t2,v1     ;   r |= !!(n & (1<<i));
    blt  v1,a1,__div2 ;   if(r >= d) {
	sub  v1,v1,a1     ;       r -= d;
	or   v0,v0,t1     ;	      q |= 1<<i;
__div2:               ;   }		    
    subiu t0,t0,1     ; i--;
    bge   t0,zero,__div1
    jr    ra
divbyzero:
    li    v0,-1
    li    v1,-1
    jr    ra
;
; Hilfsroutine zur integer-Multiplikation:
; a0, a1 = Faktoren
; Rückgabe:
; v0 = Produkt
mult:
	move	t0,a0             ; t0 = n
	move    t1,a1             ; t1 = m
	li      v0,0              ; result = 0
__mult1:
	beq	    t1,zero,__mult3   ; while (m != 0)
	andi	t2,t1,0x1         ; {
	beq	    t2,zero,__mult2   ;    if((m & 1) != 0)
	add     v0,v0,t0          ;        result = result + n;
__mult2:
	sll	    t0,t0,1           ;    n = n << 1;
    srl     t1,t1,1           ;    m = m >> 1;
    b       __mult1           ; }
__mult3:
    jr      ra                ; return result;



UART_BASE:    equ 0xffffff00
UART_RX:      equ 0
UART_STAT:    equ 4
UART_TX:      equ 8
UART_CTRL:    equ 12
STAT_RXFULL:  equ 1
STAT_TXEMPTY: equ 2

;
; Hilfsroutine zur Zeichenausgabe:
; a0 = Auszugebendes Zeichen
putc:
    la    $t0,UART_BASE
waittx:
    lw    $t1,UART_STAT($t0)     ; while(!(uart->stat & STAT_TXEMPTY));
    andi  $t1,$t1,STAT_TXEMPTY
    beq   $t1,$zero,waittx
    sw    $a0,UART_TX($t0)       ; Ein Zeichen ausgeben
	jr    $ra

;
; Hilfsroutine zur Zeicheneingabe:
; v0 = Eingelesenes Zeichen
getc:
    la    $t0,UART_BASE
waitrx:
    lw    $t1,UART_STAT($t0)     ; while(!(uart->stat & STAT_RXFULL));
    andi  $t1,$t1,STAT_RXFULL
    beq   $t1,$zero,waitrx
    lw    $v0,UART_RX($t0)       ; Ein Zeichen einlesen
	jr    $ra


;
; Hilfsroutine zur Stringausgabe:
; a0 = Zeiger auf Null-terminierten String
puts:
    subiu sp,sp,12     ; get stack space for 3xword on stack
    sw    ra,0(sp)
    sw    s0,4(sp)
    sw    a0,8(sp)     ; save registers

    move  s0,a0
output_loop:
    lb    a0,0(s0)             ; Zeichen holen
    beq   a0,zero,output_end   ; Null eingelesen?
    jal   putc                 ; Zeichen ausgeben
    addiu s0,s0,1              ; Zeiger inkrementieren
    b     output_loop          ; weiter
output_end:
    lw    a0,8(sp)     ; restore registers
    lw    s0,4(sp)
    lw    ra,0(sp)
    addiu sp,sp,12     ; free stack space
    jr    $ra
;
; Hilfsroutine zur Dezimalausgabe:
; Auszugebende Zahl ist in a0
putdec:
    subiu sp,sp,8     ; get stack space for 2xword on stack
    sw    ra,0(sp)    ; save regs
    sw    a0,4(sp)
    sltiu t0,a0,10
    bne   t0,zero,__putdec1 ; iff a0 > 9
    li    a1,10       ; a1 = 10
    jal   div         ; divide a0 / a1
    move  a0,v0       ; get quotient in a0
    jal   putdec      ; recursive call: output number / 10
    lw    a0,4(sp)    ; restore argument
__putdec1:
    li    a1,10       ; a1 = 10
    jal   div         ; divide a0 / a1
    addiu a0,v1,0x30  ; convert remainder to character
    jal   putc        ; print it
__putdec2:
    lw    ra,0(sp)
    lw    a0,4(sp)
    addiu sp,sp,8
    jr    ra

;
; Hilfsroutine zur Dezimaleingabe:
; Eingebene Zahl ist in v0
getdec:
    subiu sp,sp,12     ; get stack space for 2xword on stack
    sw    ra,0(sp)    ; save regs
    sw    s0,4(sp)
    sw    s1,8(sp)
    li    s0,0        ; result = 0;
__getdec1:
    jal   getc        ; while(isnumber(c = getc()))
	slti  t0,v0,0x30  ; {
	bne	  t0,zero,__getdec_end
	slti  t0,v0,0x3a
	beq   t0,zero,__getdec_end
	move  s1,v0       ; (save c)
    move  a0,s0
    li    a1,10
    jal   mult        ;     result *= 10;
	addiu s1,s1,-48   ;     result += c - 0x30;
    add   s0,v0,s1
    b     __getdec1
__getdec_end:
    move  v0,s0
    lw    ra,0(sp)
    lw    s0,4(sp)
    lw    s1,8(sp)
    addiu sp,sp,12
    jr    ra
;
; Daten:
;
msg1:     DB "Bitte geben Sie eine Zahl < 400 ein: "
msg2:     DB "Primzahlen von 2 bis "
msg3:     DB "\n"
msg4:     DB "Insgesamt "
msg5:     DB " Primzahlen\n"
msg6:     DB "Depp! Ich hab doch < 400 gesagt!\n"
gestrichen:
          DS 100 ; char gestrichen[400]
          DS 20 ; stack
stk:
